"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const chokidar = require("chokidar");
const ora = require("ora");
const Workspace_1 = require("../core/Workspace");
const RestClient_1 = require("../core/HTTP/RestClient");
const wipe_1 = require("./wipe");
const helpers_1 = require("../helpers");
async function asyncForEach(array, callback) {
    for (let index = 0; index < array.length; index++) {
        await callback(array[index], index, array);
    }
}
function buildFilesObject(fileObj, files, filterBy) {
    if (filterBy === 'configs') {
        fileObj.configs = files.filter((file) => {
            return !(/\//.test(file.resource.path));
        });
        files = files.filter((file) => {
            return /\//.test(file.resource.path);
        });
        return files;
    }
    fileObj[filterBy] = files.filter((file) => {
        return file.resource.path.startsWith(filterBy);
    });
    files = files.filter((file) => {
        return !file.resource.path.startsWith(filterBy);
    });
    return files;
}
async function Deploy(workspace, path) {
    let client;
    console.log(`Deploying ${workspace.name}:`);
    console.log(``);
    let spinner = ora({
        prefixText: `Deploying ${path || 'file'}...`,
        text: `reading files...`,
    }).start();
    client = new RestClient_1.default(workspace.config, workspace.name);
    try {
        await workspace.scan();
        const fileObj = {};
        let files = workspace.files;
        files = buildFilesObject(fileObj, files, 'configs');
        files = buildFilesObject(fileObj, files, 'content');
        files = buildFilesObject(fileObj, files, 'specs');
        files = buildFilesObject(fileObj, files, 'emails');
        files = buildFilesObject(fileObj, files, 'themes');
        if (files.length > 0) {
            fileObj['other files'] = files;
        }
        await asyncForEach(Object.keys(fileObj), async (fileType) => {
            if (fileType === 'specs' && workspace.config.ignoreSpecs) {
                return;
            }
            let files = fileObj[fileType];
            if (workspace.config.skipPaths && workspace.config.skipPaths.length > 0) {
                files = fileObj[fileType].filter((file) => !workspace.config.skipPaths.some((path) => file.resource.path.startsWith(path)));
            }
            spinner.prefixText = `Deploying ${fileType}`;
            for (let file of files) {
                if (path && file.location.split(path)[1] !== '') {
                    continue;
                }
                spinner.text = file.resource.path;
                await file.read();
                await client.saveFile(file.resource);
            }
            spinner.prefixText = 'Deployed';
            spinner.succeed(fileType).start();
        });
        spinner.prefixText = '';
        if (!path) {
            spinner.text = workspace.config.ignoreSpecs
                ? `Deployed all non-spec files to ${workspace.name}`
                : `Deployed all files to ${workspace.name}`;
        }
        spinner.succeed();
        console.log(``);
    }
    catch (e) {
        spinner.text = client.handleError(e);
        spinner.fail();
        throw e;
    }
}
exports.default = async (args) => {
    if (!args.preserve) {
        await wipe_1.default(args);
    }
    let workspace;
    try {
        workspace = await Workspace_1.default.init(args.workspace, args.disableSSLVerification, args.ignoreSpecs, args.skipPath);
    }
    catch (e) {
        return helpers_1.MissingWorkspaceError(args.workspace);
    }
    await Deploy(workspace);
    if (args.watch) {
        console.log(`Watching`, `${workspace.path}/*`);
        console.log(``);
        let watcher = chokidar.watch('.', {
            cwd: workspace.path,
            alwaysStat: true,
        });
        watcher.on('change', (path, stats) => {
            if (stats && stats.isFile()) {
                Deploy(workspace, path);
            }
        });
    }
};
