"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const axios_1 = require("axios");
const https = require("https");
class RestClientError extends Error {
    constructor(res, message) {
        super(message);
        this.response = res;
    }
}
exports.RestClientError = RestClientError;
class RestClient {
    constructor(workspaceConfig, workspaceName) {
        this.clientHeaders = workspaceConfig.headers || {};
        this.workspaceName = workspaceName;
        if (workspaceConfig.kongAdminUrl) {
            this.clientUrl = `${workspaceConfig.kongAdminUrl}`;
        }
        else if (workspaceConfig.upstream) {
            console.log('upstream is deprecated and will cease to function in a later release. Please use kong_admin_url (upstream url without the workspace suffix)');
            let match = workspaceConfig.upstream.match(/.*\//);
            if (!match) {
                console.log('unable to parse upstream url');
                throw new Error();
            }
            this.clientUrl = match[0];
        }
        else {
            console.log('kongAdminUrl not set');
            throw new Error();
        }
        if (workspaceConfig.kongAdminToken) {
            this.clientHeaders['Kong-Admin-Token'] = workspaceConfig.kongAdminToken;
        }
        let httpsAgent;
        if (workspaceConfig.disableSSLVerification) {
            httpsAgent = new https.Agent({
                rejectUnauthorized: false,
            });
        }
        this.client = axios_1.default.create({
            baseURL: this.clientUrl,
            headers: this.clientHeaders,
            maxContentLength: 10 * 1000000,
            httpsAgent,
        });
    }
    async getFiles(options) {
        return this.handleResponse(await this.client.get(`${this.workspaceName}/files`, options));
    }
    async getAllFiles() {
        let res = await this.client.get(`${this.workspaceName}/files`);
        let files = this.handleResponse(res);
        while (res.data.next) {
            res = await this.client.get(res.data.next);
            files = files.concat(this.handleResponse(res));
        }
        return files;
    }
    async saveFile(file, options = {}) {
        await this.client.put(`${this.workspaceName}/files/${file.path}`, file, options);
    }
    async deleteFile(file, options = {}) {
        await this.client.delete(`${this.workspaceName}/files/${file.path}`, options);
    }
    async enablePortal(options = {}) {
        await this.client.patch(`${this.workspaceName}/workspaces/${this.workspaceName}`, { config: { portal: true } }, options);
    }
    async disablePortal(options = {}) {
        await this.client.patch(`${this.workspaceName}/workspaces/${this.workspaceName}`, { config: { portal: false } }, options);
    }
    handleResponse(res) {
        if (!res.data) {
            console.log('malformed server reply');
            throw new Error();
        }
        return res.data.data;
    }
    handleError(err) {
        if (err.message === 'Request body larger than maxBodyLength limit') {
            return '\nFiles must be less than 10mb';
        }
        if (err.response && err.response.status === 401) {
            return err.message + '\n\t Make sure you have the correct admin token and RBAC settings for that token';
        }
        if (err.response && err.response.status === 404) {
            return err.message + '\n\t Make sure Portal is enabled on this workspace: \n\t "portal enable <workspace>"';
        }
        return err.message;
    }
}
exports.default = RestClient;
